package cd.casic.ci.common.pipeline.pojo.element;

import cd.casic.ci.common.pipeline.IModelTemplate;
import cd.casic.ci.common.pipeline.NameAndValue;
import cd.casic.ci.common.pipeline.enums.BuildStatus;
import cd.casic.ci.common.pipeline.enums.StartType;
import cd.casic.ci.common.pipeline.pojo.element.agent.*;
import cd.casic.ci.common.pipeline.pojo.element.market.MarketBuildAtomElement;
import cd.casic.ci.common.pipeline.pojo.element.market.MarketBuildLessAtomElement;
import cd.casic.ci.common.pipeline.pojo.element.market.MarketCheckImageElement;
import cd.casic.ci.common.pipeline.pojo.element.matrix.MatrixStatusElement;
import cd.casic.ci.common.pipeline.pojo.element.quality.QualityGateInElement;
import cd.casic.ci.common.pipeline.pojo.element.quality.QualityGateOutElement;
import cd.casic.ci.common.pipeline.pojo.element.trigger.*;
import cd.casic.ci.common.pipeline.pojo.time.BuildRecordTimeCost;
import cd.casic.ci.common.pipeline.pojo.transfer.PreStep;
import cd.casic.ci.common.pipeline.utils.ElementUtils;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;
import java.util.Map;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element
 * @Project：ops-pro
 * @name：Element
 * @Date：2025/03/26 16:17
 * @Filename：Element
 * @description：Todo
 */

@JsonTypeInfo(
        use = JsonTypeInfo.Id.NAME,
        include = JsonTypeInfo.As.PROPERTY,
        property = "@type",
        defaultImpl = EmptyElement.class
)
@JsonSubTypes({
        @JsonSubTypes.Type(value = MatrixStatusElement.class, name = MatrixStatusElement.classType),
        @JsonSubTypes.Type(value = CodeGitWebHookTriggerElement.class, name = CodeGitWebHookTriggerElement.classType),
        @JsonSubTypes.Type(value = CodeGitlabWebHookTriggerElement.class, name = CodeGitlabWebHookTriggerElement.classType),
        @JsonSubTypes.Type(value = CodeSVNWebHookTriggerElement.class, name = CodeSVNWebHookTriggerElement.classType),
        @JsonSubTypes.Type(value = CodeGithubWebHookTriggerElement.class, name = CodeGithubWebHookTriggerElement.classType),
        @JsonSubTypes.Type(value = CodeGitElement.class, name = CodeGitElement.classType),
        @JsonSubTypes.Type(value = CodeGitlabElement.class, name = CodeGitlabElement.classType),
        @JsonSubTypes.Type(value = GithubElement.class, name = GithubElement.classType),
        @JsonSubTypes.Type(value = CodeSvnElement.class, name = CodeSvnElement.classType),
        @JsonSubTypes.Type(value = LinuxScriptElement.class, name = LinuxScriptElement.classType),
        @JsonSubTypes.Type(value = WindowsScriptElement.class, name = WindowsScriptElement.classType),
        @JsonSubTypes.Type(value = ManualTriggerElement.class, name = ManualTriggerElement.classType),
        @JsonSubTypes.Type(value = RemoteTriggerElement.class, name = RemoteTriggerElement.classType),
        @JsonSubTypes.Type(value = TimerTriggerElement.class, name = TimerTriggerElement.classType),
        @JsonSubTypes.Type(value = ManualReviewUserTaskElement.class, name = ManualReviewUserTaskElement.classType),
        @JsonSubTypes.Type(value = SubPipelineCallElement.class, name = SubPipelineCallElement.classType),
        @JsonSubTypes.Type(value = MarketBuildAtomElement.class, name = MarketBuildAtomElement.classType),
        @JsonSubTypes.Type(value = MarketBuildLessAtomElement.class, name = MarketBuildLessAtomElement.classType),
        @JsonSubTypes.Type(value = MarketCheckImageElement.class, name = MarketCheckImageElement.classType),
        @JsonSubTypes.Type(value = QualityGateInElement.class, name = QualityGateInElement.classType),
        @JsonSubTypes.Type(value = QualityGateOutElement.class, name = QualityGateOutElement.classType),
        @JsonSubTypes.Type(value = CodeTGitWebHookTriggerElement.class, name = CodeTGitWebHookTriggerElement.classType),
        @JsonSubTypes.Type(value = CodeP4WebHookTriggerElement.class, name = CodeP4WebHookTriggerElement.classType)
})
@NoArgsConstructor
@Data
@Schema(title = "Element 基类")
public abstract class Element implements IModelTemplate {
    @Schema(title = "任务名称", required = false)
    private String name;
    @Schema(title = "id", required = false)
    private String id;
    @Schema(title = "状态(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private String status;
    @Schema(title = "执行次数(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private int executeCount = 1;
    @Schema(title = "是否重试(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private Boolean canRetry;
    @Schema(title = "是否跳过(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private Boolean canSkip;
    @Schema(title = "执行时间(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private Long elapsed;//"即将被timeCost代替"
    @Schema(title = "启动时间(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private Long startEpoch;//"即将被timeCost代替"
    @Schema(title = "插件原始版本(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private String originVersion;
    @Schema(title = "插件版本", required = true)
    private String version = "1.*";
    @Schema(title = "模板对比的时候是不是有变更(temporary field)", required = false)
    private Boolean templateModify;
    @Schema(title = "附加参数", required = false)
    private ElementAdditionalOptions additionalOptions;
    @Schema(title = "用户自定义ID，用于上下文键值设置", required = false)
    private String stepId;
    @Schema(title = "各项耗时", required = true)
    private BuildRecordTimeCost timeCost;
    @Schema(title = "用户自定义环境变量（插件运行时写入环境）", required = false)
    private List<NameAndValue> customEnv;
    @Schema(title = "错误类型(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private String errorType;
    @Schema(title = "错误代码(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private Integer errorCode;
    @Schema(title = "错误信息(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private String errorMsg;
    @Schema(title = "插件名称,构建结束后的快照名称(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private String atomName;
    @Schema(title = "所属插件分类代码(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private String classifyCode;
    @Schema(title = "所属插件分类名称(仅在运行构建时有用的中间参数，不要在编排保存阶段设置值）", required = false)
    private String classifyName;
    @Schema(title = "任务运行进度", required = false)
    private Double progressRate;
    private String template;
    private String ref;
    private Map<String, String> variables;
    private String asyncStatus;

    public String getAtomCode() {
        return getClassType();
    }

    public abstract String getClassType();

    public String getTaskAtom() {
        return "";
    }

    public Map<String, Object> genTaskParams() {
        return JSONUtil.parseObj(this).toBean(Map.class);
    }

    public void cleanUp() {
    }

    public PreStep transferYaml(JSONObject defaultValue) {
        return null;
    }

    public boolean elementEnabled() {
        return additionalOptions == null || additionalOptions.isEnable();
    }

    public void transformCompatibility() {
        if (additionalOptions != null && (additionalOptions.getTimeoutVar() == null || additionalOptions.getTimeoutVar().isBlank())) {
            additionalOptions.setTimeoutVar(String.valueOf(additionalOptions.getTimeout()));
        }
    }

    public String findFirstTaskIdByStartType(StartType startType) {
        return "";
    }

    public BuildStatus initStatus(boolean rerun) {
        if (!elementEnabled()) {
            return BuildStatus.SKIP;
        } else if (rerun) {
            return BuildStatus.QUEUE;
        } else if (status != null && status.equals(BuildStatus.SKIP.name())) {
            return BuildStatus.SKIP;
        } else {
            return BuildStatus.QUEUE;
        }
    }

    public void disableBySkipVar(Map<String, Object> variables) {
        ElementPostInfo elementPostInfo = additionalOptions != null ? additionalOptions.getElementPostInfo() : null;
        boolean postFlag = elementPostInfo != null;
        String elementId = postFlag ? elementPostInfo.getParentElementId() : id;
        if (variables.get(ElementUtils.getSkipElementVariableName(elementId)) != null && "true".equals(variables.get(ElementUtils.getSkipElementVariableName(elementId)))) {
            if (additionalOptions == null) {
                ElementAdditionalOptions elementAdditionalOptions = new ElementAdditionalOptions();
                elementAdditionalOptions.setRunCondition(ElementAdditionalOptions.RunCondition.PRE_TASK_SUCCESS);
                additionalOptions = elementAdditionalOptions;
            }
            additionalOptions.setEnable(false);
        }
    }

    public Map<String, Object> initTaskVar() {
        return new java.util.HashMap<>();
    }
}
