package cd.casic.ci.common.pipeline.pojo.element;

import cd.casic.ci.common.pipeline.NameAndValue;
import com.fasterxml.jackson.annotation.JsonIgnore;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.List;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.element
 * @Project：ops-pro
 * @name：ElementAdditionalOptions
 * @Date：2025/03/26 16:24
 * @Filename：ElementAdditionalOptions
 * @description：Todo
 */
@Data
@Schema(title = "插件级别流程控制模型")
@AllArgsConstructor
@NoArgsConstructor
public class ElementAdditionalOptions {
    @Schema(title = "是否启用", required = false)
    private boolean enable = true;
    @Schema(title = "是否失败时继续", required = false)
    private boolean continueWhenFailed = false;
    @Schema(title = "是否出现跳过按钮（手动继续）", required = false)
    private Boolean manualSkip;
    @Schema(title = "是否失败时重试", required = false)
    private boolean retryWhenFailed = false;
    @Schema(title = "重试计数", required = false)
    private int retryCount = 0;
    @Schema(title = "是否允许手动重试", required = false)
    private boolean manualRetry = true;
    @Schema(title = "超时分钟", required = false)
    private Long timeout = 100L;
    @Schema(title = "新的执行的超时时间，支持变量(分钟Minutes)，出错则取timeout的值", required = false)
    private String timeoutVar;
    @JsonIgnore
    private boolean change = false;
    @Schema(title = "执行条件", required = false)
    private RunCondition runCondition;
    @Schema(title = "是否配置前置暂停", required = false)
    private Boolean pauseBeforeExec = false;
    @Schema(title = "订阅暂停通知用户", required = false)
    private String subscriptionPauseUser = "";
    @Schema(title = "", required = false)
    private String otherTask;
    @Schema(title = "自定义变量", required = false)
    private List<NameAndValue> customVariables;
    @Schema(title = "自定义条件", required = false)
    private String customCondition;
    @Schema(title = "插件post信息", required = false)
    private ElementPostInfo elementPostInfo;
    @Schema(title = "是否设置自定义环境变量", required = false)
    @Deprecated
    private Boolean enableCustomEnv = true;
    @Schema(title = "用户自定义环境变量（插件运行时写入环境）", required = false)
    @Deprecated
    private List<NameAndValue> customEnv;

    public ElementAdditionalOptions(boolean continueWhenFailed, Long timeout, RunCondition runCondition, ElementPostInfo elementPostInfo) {
        this.continueWhenFailed = continueWhenFailed;
        this.timeout = timeout;
        this.runCondition = runCondition;
        this.elementPostInfo = elementPostInfo;
    }

    public enum RunCondition {
        PRE_TASK_SUCCESS,
        PRE_TASK_FAILED_BUT_CANCEL,
        PRE_TASK_FAILED_EVEN_CANCEL,
        PRE_TASK_FAILED_ONLY,
        OTHER_TASK_RUNNING,
        CUSTOM_VARIABLE_MATCH,
        CUSTOM_VARIABLE_MATCH_NOT_RUN,
        CUSTOM_CONDITION_MATCH,
        PARENT_TASK_CANCELED_OR_TIMEOUT,
        PARENT_TASK_FINISH
    }
}
