package cd.casic.pipeline.dispatch.api;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.dispatch.pojo.thirdpartyagent.AgentBuildInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "SERVICE_AGENT", description = "服务-Agent")
@RestController
@RequestMapping("/service/agents")
public interface ServiceAgentResource {

    @Operation(summary = "获取agent构建信息")
    @GetMapping("/{agentId}/listBuilds")
    PageResult<AgentBuildInfo> listAgentBuild(
            @Parameter(description = "agent Hash ID", required = true)
            @PathVariable("agentId") String agentId,
            @Parameter(description = "筛选此状态，支持4种输入(QUEUE,RUNNING,DONE,FAILURE)", required = false)
            @RequestParam("status") String status,
            @Parameter(description = "筛选此pipelineId", required = false)
            @RequestParam("pipelineId") String pipelineId,
            @Parameter(description = "第几页", required = false)
            @RequestParam("page") Integer page,
            @Parameter(description = "每页条数", required = false)
            @RequestParam("pageSize") Integer pageSize
    );

    @Operation(summary = "批量获取构建机最近执行记录")
    @GetMapping("listLatestBuildPipelines")
    CommonResult<List<AgentBuildInfo>> listLatestBuildPipelines(
            @Parameter(description = "agent Hash ID", required = true)
            @RequestParam("agentIds") List<String> agentIds
    );

    @Operation(summary = "获取agent登录调试url")
    @GetMapping("/docker/debug/url")
    CommonResult<String> getDockerDebugUrl(
            @RequestParam("userId") String userId,
            @RequestParam("projectId") String projectId,
            @RequestParam("pipelineId") String pipelineId,
            @RequestParam("buildId") String buildId,
            @RequestParam("vmSeqId") String vmSeqId
    );
}