package cd.casic.ci.log.api;


import cd.casic.ci.log.dal.pojo.EndPageQueryLogs;
import cd.casic.ci.log.dal.pojo.PageQueryLogs;
import cd.casic.ci.log.dal.pojo.QueryLogs;
import cd.casic.ci.log.dal.pojo.enums.LogType;
import cd.casic.framework.commons.pojo.CommonResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.*;;

@Tag(name = "用户-日志资源")
@RestController
@RequestMapping("/app/logs")
public interface AppLogResource {

    @Operation(summary = "根据构建ID获取初始化所有日志")
    @GetMapping("/{projectId}/{pipelineId}/{buildId}/")
    CommonResult<PageQueryLogs> getInitLogs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam(value = "debug", defaultValue = "false")
            boolean debug,
            @Parameter(description = "过滤日志级别", required = false)
            @RequestParam(value = "logType", defaultValue = "null")
            LogType logType,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam(value = "tag")
            String tag,
            @Parameter(description = "指定subTag", required = false)
            @RequestParam("subTag")
            String subTag,
            @Parameter(description = "对应jobId", required = false)
            @RequestParam(value = "jobId")
            String jobId,
            @Parameter(description = "执行次数", required = false)
            @RequestParam(value = "executeCount")
            int executeCount,
            @Parameter(description = "第几页", required = false)
            @RequestParam("page")
            int page,
            @Parameter(description = "每页条数", required = false)
            @RequestParam("pageSize")
            int pageSize
    );

    @Operation(summary = "获取更多日志")
    @GetMapping("/{projectId}/{pipelineId}/{buildId}/more")
    CommonResult<QueryLogs> getMoreLogs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam(value = "debug", defaultValue = "false")
            boolean debug,
            @Parameter(description = "过滤日志级别", required = false)
            @RequestParam(value = "logType", defaultValue = "null")
            LogType logType,
            @Parameter(description = "日志行数", required = false)
            @RequestParam(value = "num", defaultValue = "100")
            int num,
            @Parameter(description = "是否正序输出", required = false)
            @RequestParam(value = "fromStart", defaultValue = "true")
            boolean fromStart,
            @Parameter(description = "起始行号", required = true)
            @RequestParam(value = "start")
            long start,
            @Parameter(description = "结尾行号", required = true)
            @RequestParam(value = "end")
            long end,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam(value = "tag")
            String tag,
            @Parameter(description = "指定subTag", required = false)
            @RequestParam("subTag")
            String subTag,
            @Parameter(description = "对应jobId", required = false)
            @RequestParam(value = "jobId")
            String jobId,
            @Parameter(description = "执行次数", required = false)
            @RequestParam(value = "executeCount")
            int executeCount
    );

    @Operation(summary = "获取某行后的日志")
    @GetMapping("/{projectId}/{pipelineId}/{buildId}/after")
    CommonResult<QueryLogs> getAfterLogs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "起始行号", required = true)
            @RequestParam("start")
            long start,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam(value = "debug", defaultValue = "false")
            boolean debug,
            @Parameter(description = "过滤日志级别", required = false)
            @RequestParam(value = "logType", defaultValue = "null")
            LogType logType,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam("tag")
            String tag,
            @Parameter(description = "指定subTag", required = false)
            @RequestParam("subTag")
            String subTag,
            @Parameter(description = "对应jobId", required = false)
            @RequestParam("jobId")
            String jobId,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            int executeCount
    );

    @Operation(summary = "获取某行前指定行数的日志")
    @GetMapping("/{projectId}/{pipelineId}/{buildId}/before")
    CommonResult<QueryLogs> getBeforeLogs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "结束行号", required = true)
            @RequestParam("end")
            long end,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam(value = "debug", defaultValue = "false")
            boolean debug,
            @Parameter(description = "过滤日志级别", required = false)
            @RequestParam(value = "logType", defaultValue = "null")
            LogType logType,
            @Parameter(description = "返回日志条数", required = false)
            @RequestParam("size")
            int size,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam("tag")
            String tag,
            @Parameter(description = "指定subTag", required = false)
            @RequestParam("subTag")
            String subTag,
            @Parameter(description = "对应jobId", required = false)
            @RequestParam("jobId")
            String jobId,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            int executeCount
    );

    @Operation(summary = "根据构建ID获取初始化最后日志")
    @GetMapping("/{projectId}/{pipelineId}/{buildId}/end")
    CommonResult<EndPageQueryLogs> getEndLogsPage(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(value = AUTH_HEADER_USER_ID, defaultValue = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            String userId,
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "返回日志条数", required = true)
            @RequestParam("size")
            int size,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam(value = "debug", defaultValue = "false")
            boolean debug,
            @Parameter(description = "过滤日志级别", required = false)
            @RequestParam(value = "logType", defaultValue = "null")
            LogType logType,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam("tag")
            String tag,
            @Parameter(description = "指定subTag", required = false)
            @RequestParam("subTag")
            String subTag,
            @Parameter(description = "对应jobId", required = false)
            @RequestParam("jobId")
            String jobId,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            int executeCount
    );

    @Operation(summary = "根据构建ID获取初始化底部指定行数的日志")
    @GetMapping("/{projectId}/{pipelineId}/{buildId}/bottom")
    CommonResult<QueryLogs> getBottomLogs(
            @Parameter(description = "用户ID", required = true, example = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            @RequestHeader(value = AUTH_HEADER_USER_ID, defaultValue = AUTH_HEADER_USER_ID_DEFAULT_VALUE)
            String userId,
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "流水线ID", required = true)
            @PathVariable("pipelineId")
            String pipelineId,
            @Parameter(description = "构建ID", required = true)
            @PathVariable("buildId")
            String buildId,
            @Parameter(description = "是否包含调试日志", required = false)
            @RequestParam(value = "debug", defaultValue = "false")
            boolean debug,
            @Parameter(description = "过滤日志级别", required = false)
            @RequestParam(value = "logType", defaultValue = "null")
            LogType logType,
            @Parameter(description = "返回日志条数", required = false)
            @RequestParam("size")
            int size,
            @Parameter(description = "对应elementId", required = false)
            @RequestParam("tag")
            String tag,
            @Parameter(description = "指定subTag", required = false)
            @RequestParam("subTag")
            String subTag,
            @Parameter(description = "对应jobId", required = false)
            @RequestParam("jobId")
            String jobId,
            @Parameter(description = "执行次数", required = false)
            @RequestParam("executeCount")
            int executeCount
    );

    @Operation(summary = "测试接口")
    @GetMapping("test")
    CommonResult<String> test(
            @Parameter(description = "构建的buildId", required = true, example = "1")
            @RequestParam("buildId")
            String buildId,
            @Parameter(description = "标签名称", required = true, example = "java")
            @RequestParam("tagName")
            String tagName,
            @Parameter(description = "要添加子标签", required = true, example = "spring")
            @RequestParam("subTag")
            String subTag
    );
}
