package cd.casic.ci.process.engine.scheduler.service.impl;

import cd.casic.ci.process.engine.enums.ContextStateEnum;
import cd.casic.ci.process.engine.scheduler.config.PipelineSchedulerConfig;
import cd.casic.ci.process.engine.scheduler.dao.PipelineSchedulingPropertiesDao;
import cd.casic.ci.process.engine.scheduler.dateObject.PipelineSchedulingProperties;
import cd.casic.ci.process.engine.scheduler.service.PipelineSchedulingPropertiesService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName PipelineSchedulingPropertiesServiceImpl
 * @Date: 2025/5/26 16:49
 * @Description:
 */

@Service
@Slf4j
public class PipelineSchedulingPropertiesServiceImpl extends ServiceImpl<PipelineSchedulingPropertiesDao, PipelineSchedulingProperties> implements PipelineSchedulingPropertiesService {

    @Resource
    private PipelineSchedulingPropertiesDao taskRepository;

    @Resource
    private PipelineSchedulerConfig pipelineSchedulerConfig;

    public List<PipelineSchedulingProperties> getAllTasks() {
        return taskRepository.selectList(null);
    }

    public void addTask(PipelineSchedulingProperties task) {
        if (ContextStateEnum.RUNNING.getCode() == Integer.parseInt(task.getStatus())) {
            pipelineSchedulerConfig.addTask(task.getPipelineId(), () -> {
            },task.getCronExpression());
        }
        taskRepository.insert(task);
    }

    public void updateTask(PipelineSchedulingProperties task) {
        PipelineSchedulingProperties old = taskRepository.selectById(task.getId());
        if (old != null) {
            pipelineSchedulerConfig.updateTask(old.getPipelineId(), task.getCronExpression());
        }
        taskRepository.updateById(task);
    }

    public void deleteTask(String taskId) {
        pipelineSchedulerConfig.removeTask(taskId);
        taskRepository.delete(new QueryWrapper<PipelineSchedulingProperties>().eq("task_id", taskId));
    }

    public void startTask(String taskId) {
        pipelineSchedulerConfig.startTask(taskId);
        taskRepository.update(null, new UpdateWrapper<PipelineSchedulingProperties>()
                .set("status", String.valueOf(ContextStateEnum.RUNNING.getCode())).eq("task_id", taskId));
    }

    public void stopTask(String taskId) {
        pipelineSchedulerConfig.stopTask(taskId);
        taskRepository.update(null, new UpdateWrapper<PipelineSchedulingProperties>()
                .set("status", String.valueOf(ContextStateEnum.STOP.getCode())).eq("task_id", taskId));
    }
}
