package cd.casic.ci.process.engine.worker;


import cd.casic.ci.process.constant.CommandConstant;
import cd.casic.ci.process.engine.constant.EngineRuntimeConstant;
import cd.casic.ci.process.engine.enums.ContextStateEnum;
import cd.casic.ci.process.engine.manager.LoggerManager;
import cd.casic.ci.process.engine.manager.RunContextManager;
import cd.casic.ci.process.engine.runContext.BaseRunContext;

import cd.casic.ci.process.engine.runContext.TaskRunContext;
import cd.casic.ci.process.enums.MachineSystemEnum;
import cd.casic.ci.process.process.dataObject.base.PipBaseElement;
import cd.casic.ci.process.process.dataObject.log.PipTaskLog;
import cd.casic.ci.process.process.dataObject.machine.MachineInfo;
import cd.casic.ci.process.process.service.machine.MachineInfoService;

import cd.casic.ci.process.ssh.SshClient;
import cd.casic.ci.process.ssh.SshClientFactory;
import cd.casic.ci.process.ssh.WinRMHelper;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import jakarta.annotation.Resource;

import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;

import javax.swing.text.StringContent;
import java.util.Arrays;
import java.util.List;


@Data
@Slf4j
public abstract class BaseWorker implements Runnable{
    // 一些属性
    @Resource
    private RunContextManager contextManager;
    private String contextKey;
    @Resource
    private MachineInfoService machineInfoService;
    @Resource
    private LoggerManager loggerManager;


    @Override
    public void run() {
        if (StringUtils.isEmpty(contextKey)) {
            log.error("请设置当前worker的contextKey");
            return;
        }
        doWorker(contextKey);
    }

    public void doWorker(String contextKey){
        BaseRunContext context = contextManager.getContext(contextKey);
        if (context instanceof TaskRunContext taskRunContext){
            try {
                taskRunContext.changeContextState(ContextStateEnum.READY);
                taskRunContext.changeContextState(ContextStateEnum.RUNNING);
                PipBaseElement contextDef = taskRunContext.getContextDef();
                PipTaskLog pipTaskLog = new PipTaskLog();
                pipTaskLog.setId(contextDef.getId());
                pipTaskLog.setContent("");
                taskRunContext.getLocalVariables().put(EngineRuntimeConstant.LOG_KEY,pipTaskLog);
                execute(taskRunContext);
            } catch (Exception e) {
                log.error("================worker执行报错：",e);
                taskRunContext.changeContextState(ContextStateEnum.BAD_ENDING);
                return;
            }
            // TODO 执行结束修改context的state,并且通知父类
            taskRunContext.changeContextState(ContextStateEnum.HAPPY_ENDING);
        }
    }
    public void append(BaseRunContext context, String content){
        if (context instanceof TaskRunContext taskRunContext) {
            loggerManager.append(taskRunContext.getContextDef().getId(),content);
        }
    }
    public abstract void execute(TaskRunContext context);
    public void toBadEnding(){
        throw new ServiceException(GlobalErrorCodeConstants.PIPELINE_ERROR.getCode(),"");
    }
}
