package cd.casic.ci.ticket.dal.enums;

import java.util.HashMap;
import java.util.Map;

public enum CredentialType {
    PASSWORD, // v1 = password, v2=v3=v4=null
    ACCESSTOKEN, // v1 = access_token
    OAUTHTOKEN, // v1 = oauth_token
    USERNAME_PASSWORD, // v1 = username, v2 = password, other = null
    SECRETKEY, // v1 = secretKey, other = null
    APPID_SECRETKEY, // v1 = appId, v2 = secretKey, other = null
    SSH_PRIVATEKEY, // v1 = privateKey, v2=passphrase?
    TOKEN_SSH_PRIVATEKEY, // v1 = token, v2 = privateKey, v3=passphrase?
    TOKEN_USERNAME_PASSWORD, // v1 = token, v2 = username, v3=password
    COS_APPID_SECRETID_SECRETKEY_REGION, // v1 = cosappId, v2 = secretId, v3 = secretKey, v4 = region
    MULTI_LINE_PASSWORD; // 密码中有换行符 v1 = password, v2=v3=v4=null

    public static Map<String, String> getKeyMap(String credentialType) {
        Map<String, String> keyMap = new HashMap<>();
        switch (credentialType) {
            case "PASSWORD":
                keyMap.put("v1", "password");
                break;
            case "ACCESSTOKEN":
                keyMap.put("v1", "access_token");
                break;
            case "OAUTHTOKEN":
                keyMap.put("v1", "oauth_token");
                break;
            case "USERNAME_PASSWORD":
                keyMap.put("v1", "username");
                keyMap.put("v2", "password");
                break;
            case "SECRETKEY":
                keyMap.put("v1", "secretKey");
                break;
            case "APPID_SECRETKEY":
                keyMap.put("v1", "appId");
                keyMap.put("v2", "secretKey");
                break;
            case "SSH_PRIVATEKEY":
                keyMap.put("v1", "privateKey");
                keyMap.put("v2", "passphrase");
                break;
            case "TOKEN_SSH_PRIVATEKEY":
                keyMap.put("v1", "token");
                keyMap.put("v2", "privateKey");
                keyMap.put("v3", "passphrase");
                break;
            case "TOKEN_USERNAME_PASSWORD":
                keyMap.put("v1", "token");
                keyMap.put("v2", "username");
                keyMap.put("v3", "password");
                break;
            case "COS_APPID_SECRETID_SECRETKEY_REGION":
                keyMap.put("v1", "cosappId");
                keyMap.put("v2", "secretId");
                keyMap.put("v3", "secretKey");
                keyMap.put("v4", "region");
                break;
            case "MULTI_LINE_PASSWORD":
                keyMap.put("v1", "password");
                break;
        }
        return keyMap;
    }
}
